package com.baidu.translate.asr.demo.util;

import android.os.Build;
import android.os.Environment;

import com.baidu.translate.asr.demo.BuildConfig;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.Thread.UncaughtExceptionHandler;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * 自动保存崩溃日志. <br/>
 * 崩溃日志会自动保存在 <b>/sdcard/Android/data/应用程序包名/crash</b>目录下.<br/>
 * 如果没有外置存储卡, 则会保存在 <b>/data/data/应用程序包名/crash</b>目录下
 *
 * @author wangjingtao
 */
public class CrashHandler implements UncaughtExceptionHandler {

    // 系统默认的UncaughtException处理类
    private static UncaughtExceptionHandler mDefaultHandler;

    // CrashHandler实例
    private static CrashHandler instance;

    // 崩溃日志的扩展名
    private static final String CRASH_REPORTER_EXTENSION = ".crash.log";

    private CrashHandler() {

    }

    /**
     * 在Activity或Application中初始化均可以
     */
    public static void init() {
        if (instance == null) {
            instance = new CrashHandler();
        }

        mDefaultHandler = Thread.getDefaultUncaughtExceptionHandler();
        Thread.setDefaultUncaughtExceptionHandler(instance);
    }

    @Override
    public void uncaughtException(Thread thread, Throwable throwable) {
        handleException(throwable);
        if (mDefaultHandler != null) {
            mDefaultHandler.uncaughtException(thread, throwable);
        }
        System.exit(0);
    }

    private boolean handleException(Throwable ex) {
        if (ex == null) {
            return true;
        }
        ex.printStackTrace();
        return saveCrashInfoToFile(ex) != null;
    }

    private String saveCrashInfoToFile(Throwable ex) {
        Writer info = new StringWriter();
        PrintWriter printWriter = new PrintWriter(info);
        ex.printStackTrace(printWriter);

        Throwable cause = ex.getCause();
        while (cause != null) {
            cause.printStackTrace(printWriter);
            cause = cause.getCause();
        }
        // 系统信息
        printWriter.append("\r\nVersionName=");
        printWriter.append(BuildConfig.VERSION_NAME);
        printWriter.append("\r\nVersionCode=");
        printWriter.append(String.valueOf(BuildConfig.VERSION_CODE));

        // 使用反射来收集设备信息.在Build类中包含各种设备信息,
        // 例如: 系统版本号,设备生产商 等帮助调试程序的有用信息
        // 具体信息请参考后面的截图
        Field[] fields = Build.class.getDeclaredFields();
        if (fields != null) {
            for (Field field : fields) {
                try {
                    field.setAccessible(true);
                    printWriter.append("\r\n");
                    printWriter.append(field.getName());
                    printWriter.append(": ");
                    printWriter.append(String.valueOf(field.get(null)));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

        String result = info.toString();
        printWriter.close();
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss", Locale.US);
            File file = new File(getCrashDir(), "crash_" + sdf.format(new Date()) + CRASH_REPORTER_EXTENSION);
            if (!file.getParentFile().exists() && !file.getParentFile().mkdirs()) {
                return null;
            }
            FileWriter writer = new FileWriter(file, true);
            writer.write(result);
            writer.flush();
            writer.close();
            return file.getAbsolutePath();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    private String getCrashDir() {
        if (Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_MOUNTED)) {
            return Environment.getExternalStorageDirectory().getAbsolutePath() + "/Android/data/"
                    + BuildConfig.APPLICATION_ID + "/crash/";
        }

        return Environment.getDataDirectory().getAbsolutePath() + "/data/" + BuildConfig.APPLICATION_ID + "/crash/";
    }

}
